package com.sun.xml.internal.bind.v2;

import java.io.InputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.List;
import java.io.IOException;
import java.util.StringTokenizer;
import com.sun.istack.internal.FinalArrayList;
import com.sun.xml.internal.bind.api.TypeReference;
import com.sun.xml.internal.bind.v2.runtime.JAXBContextImpl;
import javax.xml.bind.JAXBException;
import com.sun.xml.internal.bind.v2.util.TypeCast;
import java.util.Collection;
import com.sun.xml.internal.bind.api.JAXBRIContext;
import com.sun.xml.internal.bind.v2.model.annotation.RuntimeAnnotationReader;
import java.util.logging.Level;
import com.sun.xml.internal.bind.Util;
import java.util.Collections;
import java.util.HashMap;
import javax.xml.bind.JAXBContext;
import java.util.Map;

public class ContextFactory
{
    public static final String USE_JAXB_PROPERTIES = "_useJAXBProperties";
    
    public ContextFactory() {
    }
    
    public static JAXBContext createContext(Class[] classes, Map<String, Object> properties) throws JAXBException {
        if (properties == null) {
            properties = Collections.emptyMap();
        }
        else {
            properties = new HashMap(properties);
        }
        String defaultNsUri = (String)getPropertyValue(properties, "com.sun.xml.internal.bind.defaultNamespaceRemap", String.class);
        Boolean c14nSupport = (Boolean)getPropertyValue(properties, "com.sun.xml.internal.bind.c14n", Boolean.class);
        if (c14nSupport == null) {
            c14nSupport = Boolean.valueOf(false);
        }
        Boolean disablesecurityProcessing = (Boolean)getPropertyValue(properties, "com.sun.xml.internal.bind.disableXmlSecurity", Boolean.class);
        if (disablesecurityProcessing == null) {
            disablesecurityProcessing = Boolean.valueOf(false);
        }
        Boolean allNillable = (Boolean)getPropertyValue(properties, "com.sun.xml.internal.bind.treatEverythingNillable", Boolean.class);
        if (allNillable == null) {
            allNillable = Boolean.valueOf(false);
        }
        Boolean retainPropertyInfo = (Boolean)getPropertyValue(properties, "retainReferenceToInfo", Boolean.class);
        if (retainPropertyInfo == null) {
            retainPropertyInfo = Boolean.valueOf(false);
        }
        Boolean supressAccessorWarnings = (Boolean)getPropertyValue(properties, "supressAccessorWarnings", Boolean.class);
        if (supressAccessorWarnings == null) {
            supressAccessorWarnings = Boolean.valueOf(false);
        }
        Boolean improvedXsiTypeHandling = (Boolean)getPropertyValue(properties, "com.sun.xml.internal.bind.improvedXsiTypeHandling", Boolean.class);
        if (improvedXsiTypeHandling == null) {
            String improvedXsiSystemProperty = Util.getSystemProperty("com.sun.xml.internal.bind.improvedXsiTypeHandling");
            if (improvedXsiSystemProperty == null) {
                improvedXsiTypeHandling = Boolean.valueOf(true);
            }
            else {
                improvedXsiTypeHandling = Boolean.valueOf(improvedXsiSystemProperty);
            }
        }
        Boolean xmlAccessorFactorySupport = (Boolean)getPropertyValue(properties, "com.sun.xml.internal.bind.XmlAccessorFactory", Boolean.class);
        if (xmlAccessorFactorySupport == null) {
            xmlAccessorFactorySupport = Boolean.valueOf(false);
            Util.getClassLogger().log(Level.FINE, "Property com.sun.xml.internal.bind.XmlAccessorFactoryis not active.  Using JAXB's implementation");
        }
        RuntimeAnnotationReader ar = (RuntimeAnnotationReader)getPropertyValue(properties, JAXBRIContext.ANNOTATION_READER, RuntimeAnnotationReader.class);
        Collection<TypeReference> tr = (Collection)getPropertyValue(properties, "com.sun.xml.internal.bind.typeReferences", Collection.class);
        if (tr == null) {
            tr = Collections.emptyList();
        }
        Map<Class, Class> subclassReplacements;
        try {
            subclassReplacements = TypeCast.checkedCast((Map)getPropertyValue(properties, "com.sun.xml.internal.bind.subclassReplacements", Map.class), Class.class, Class.class);
        }
        catch (ClassCastException e) {
            throw new JAXBException(Messages.INVALID_TYPE_IN_MAP.format(new Object[0]), e);
        }
        if (!properties.isEmpty()) {
            throw new JAXBException(Messages.UNSUPPORTED_PROPERTY.format(new Object[] { properties.keySet().iterator().next() }));
        }
        JAXBContextImpl.JAXBContextBuilder builder = new JAXBContextImpl.JAXBContextBuilder();
        builder.setClasses(classes);
        builder.setTypeRefs(tr);
        builder.setSubclassReplacements(subclassReplacements);
        builder.setDefaultNsUri(defaultNsUri);
        builder.setC14NSupport((boolean)c14nSupport);
        builder.setAnnotationReader(ar);
        builder.setXmlAccessorFactorySupport((boolean)xmlAccessorFactorySupport);
        builder.setAllNillable((boolean)allNillable);
        builder.setRetainPropertyInfo((boolean)retainPropertyInfo);
        builder.setSupressAccessorWarnings((boolean)supressAccessorWarnings);
        builder.setImprovedXsiTypeHandling((boolean)improvedXsiTypeHandling);
        builder.setDisableSecurityProcessing((boolean)disablesecurityProcessing);
        return builder.build();
    }
    
    private static <T> T getPropertyValue(Map<String, Object> properties, String keyName, Class<T> type) throws JAXBException {
        Object o = properties.get(keyName);
        if (o == null) {
            return null;
        }
        properties.remove(keyName);
        if (!type.isInstance(o)) {
            throw new JAXBException(Messages.INVALID_PROPERTY_VALUE.format(new Object[] { keyName, o }));
        }
        return type.cast(o);
    }
    
    @Deprecated
    public static JAXBRIContext createContext(Class[] classes, Collection<TypeReference> typeRefs, Map<Class, Class> subclassReplacements, String defaultNsUri, boolean c14nSupport, RuntimeAnnotationReader ar, boolean xmlAccessorFactorySupport, boolean allNillable, boolean retainPropertyInfo) throws JAXBException {
        return createContext(classes, typeRefs, subclassReplacements, defaultNsUri, c14nSupport, ar, xmlAccessorFactorySupport, allNillable, retainPropertyInfo, false);
    }
    
    @Deprecated
    public static JAXBRIContext createContext(Class[] classes, Collection<TypeReference> typeRefs, Map<Class, Class> subclassReplacements, String defaultNsUri, boolean c14nSupport, RuntimeAnnotationReader ar, boolean xmlAccessorFactorySupport, boolean allNillable, boolean retainPropertyInfo, boolean improvedXsiTypeHandling) throws JAXBException {
        JAXBContextImpl.JAXBContextBuilder builder = new JAXBContextImpl.JAXBContextBuilder();
        builder.setClasses(classes);
        builder.setTypeRefs(typeRefs);
        builder.setSubclassReplacements(subclassReplacements);
        builder.setDefaultNsUri(defaultNsUri);
        builder.setC14NSupport(c14nSupport);
        builder.setAnnotationReader(ar);
        builder.setXmlAccessorFactorySupport(xmlAccessorFactorySupport);
        builder.setAllNillable(allNillable);
        builder.setRetainPropertyInfo(retainPropertyInfo);
        builder.setImprovedXsiTypeHandling(improvedXsiTypeHandling);
        return builder.build();
    }
    
    public static JAXBContext createContext(String contextPath, ClassLoader classLoader, Map<String, Object> properties) throws JAXBException {
        FinalArrayList<Class> classes = new FinalArrayList();
        StringTokenizer tokens = new StringTokenizer(contextPath, ":");
        while (tokens.hasMoreTokens()) {
            boolean foundObjectFactory;
            boolean foundJaxbIndex = foundObjectFactory = false;
            String pkg = tokens.nextToken();
            try {
                Class<?> o = classLoader.loadClass(pkg + ".ObjectFactory");
                classes.add(o);
                foundObjectFactory = true;
            }
            catch (ClassNotFoundException ex) {}
            List<Class> indexedClasses;
            try {
                indexedClasses = loadIndexedClasses(pkg, classLoader);
            }
            catch (IOException e) {
                throw new JAXBException(e);
            }
            if (indexedClasses != null) {
                classes.addAll(indexedClasses);
                foundJaxbIndex = true;
            }
            if (!foundObjectFactory && !foundJaxbIndex) {
                throw new JAXBException(Messages.BROKEN_CONTEXTPATH.format(new Object[] { pkg }));
            }
            continue;
        }
        return createContext((Class[])classes.toArray(new Class[classes.size()]), properties);
    }
    
    private static List<Class> loadIndexedClasses(String pkg, ClassLoader classLoader) throws IOException, JAXBException {
        String resource = pkg.replace('.', '/') + "/jaxb.index";
        InputStream resourceAsStream = classLoader.getResourceAsStream(resource);
        if (resourceAsStream == null) {
            return null;
        }
        BufferedReader in = new BufferedReader(new InputStreamReader(resourceAsStream, "UTF-8"));
        try {
            FinalArrayList<Class> classes = new FinalArrayList();
            String className = in.readLine();
            while (className != null) {
                className = className.trim();
                if (className.startsWith("#") || className.length() == 0) {
                    className = in.readLine();
                }
                else {
                    if (className.endsWith(".class")) {
                        throw new JAXBException(Messages.ILLEGAL_ENTRY.format(new Object[] { className }));
                    }
                    try {
                        classes.add(classLoader.loadClass(pkg + 46 + className));
                    }
                    catch (ClassNotFoundException e) {
                        throw new JAXBException(Messages.ERROR_LOADING_CLASS.format(new Object[] { className, resource }), e);
                    }
                    className = in.readLine();
                }
            }
            return classes;
        }
        finally {
            in.close();
        }
    }
}
